/*
    Project: TCTEMP
    Version: V1.11
    File   : tc_pkcs_5_2.c

    Copyright (c) 2006-2008 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tomcrypt.h"
#include "tc_priv.h"

/*
 * Key deriving function (PBKDF2 as specified in PKCS #5 v2)
 */

BOOL __stdcall TC_DeriveKey(void *pKey, size_t cbKeySize,
  const void *pPassword, size_t cbPasswordSize,
  const void *pSalt, size_t cbSaltSize,
  unsigned uHash, unsigned uIterationCount,
  PPKCS5_2_BUFFER pBuffer)
{
    const struct ltc_hash_descriptor *pdHash;
    unsigned long  ulKeySize;

    switch (uHash) {
#ifdef RIPEMD160
    case TC_RIPEMD160:
        if (uIterationCount == 0) {
            uIterationCount = TC_RIPEMD160_ITERATION_COUNT;
        }
        pdHash = &rmd160_desc;
        break;
#endif
#ifdef SHA512
    case TC_SHA512:
        if (uIterationCount == 0) {
            uIterationCount = TC_SHA512_ITERATION_COUNT;
        }
        pdHash = &sha512_desc;
        break;
#endif
#ifdef WHIRLPOOL
    case TC_WHIRLPOOL:
        if (uIterationCount == 0) {
            uIterationCount = TC_WHIRLPOOL_ITERATION_COUNT;
        }
        pdHash = &whirlpool_desc;
        break;
#endif
    default:
        return FALSE;
    }

    ulKeySize = (unsigned long) cbKeySize;
    if (tc_pkcs_5_alg2(pPassword, (unsigned long)cbPasswordSize, pSalt,
      (unsigned long)cbSaltSize, uIterationCount, pdHash, pKey, &ulKeySize, pBuffer) !=
      CRYPT_OK) {
        return FALSE;
    }

    return ulKeySize == cbKeySize;
}

BOOL __stdcall TC_DeriveNonWeakKey(PTC_KEY pKey,
  const void *pPassword, size_t cbPasswordSize,
  void *pSalt, size_t cbSaltSize,
  unsigned uHash, unsigned uIterationCount,
  unsigned uCipher, PPKCS5_2_BUFFER pBuffer)
{
    unsigned __int8 *pbSalt = (unsigned __int8 *) pSalt;
    unsigned i;

    for (i = 256; --i; ) {
        if (!TC_DeriveKey(pKey, sizeof(TC_KEY), pPassword,
          cbPasswordSize, pSalt, cbSaltSize,
          uHash, uIterationCount, pBuffer)) {
            return FALSE;
        }
        if (!TC_KeyIsWeak(pKey, uCipher)) {
            return TRUE;
        }

        if (cbSaltSize == 0) {
            return FALSE;
        }

        pbSalt[0]++;
    }

    return FALSE;
}
